unit uFxMemoryManager;
//==============================================================================
// / TStrings   
//
// :  .                                                05.03.2011
// e-mail: albr@ukr.net; icq: 99258304; skype: albr78;
//==============================================================================

interface
uses
  Windows, Classes, dialogs;

type
  TFxMemoryFile = class
  private
    FFileMapObj: THandle; // FaleMapping
    FBaseAddress: pointer; //""    
    FMutex: THandle; //      
    FIsNewFile: boolean;
  public
    constructor Create(aFileName: string);
    destructor Destroy(); override;
  public
    procedure SaveList(aStrings: TStrings);
    procedure LoadList(aStrings: TStrings);
    procedure GetList(aStrings: TStrings);
  public
    property IsNewFile: boolean read FIsNewFile;
  end;

implementation
uses
  SysUtils;

function DoCreateMutex(AName: string): Cardinal;
var
  SD: TSecurityDescriptor;
  SA: TSecurityAttributes;
  pSA: PSecurityAttributes;
begin
  if not InitializeSecurityDescriptor(@SD, SECURITY_DESCRIPTOR_REVISION) then
    raise Exception.CreateFmt('Error InitializeSecurityDescriptor: %s', [SysErrorMessage(GetLastError)]);

  SA.nLength:=SizeOf(TSecurityAttributes);
  SA.lpSecurityDescriptor:=@SD;
  SA.bInheritHandle:=False;

  if not SetSecurityDescriptorDacl(SA.lpSecurityDescriptor, True, nil, False) then
    raise Exception.CreateFmt('Error SetSecurityDescriptorDacl: %s', [SysErrorMessage(GetLastError)]);

  pSA := @SA;
  Result := CreateMutex(pSA, False, PChar(AName));

  if Result = 0 then
    raise Exception.CreateFmt('Error CreateMutex: %s', [SysErrorMessage(GetLastError)]);
end;

{ TFxMemoryFile }
constructor TFxMemoryFile.Create(aFileName: string);
//==============================================================================
// 
//
// :  .                                                05.03.2011
// e-mail: albr@ukr.net; icq: 99258304; skype: albr78;
//==============================================================================
begin
  FMutex := DoCreateMutex(aFileName);

  FFileMapObj := CreateFileMapping(MAXDWORD, Nil, PAGE_READWRITE, 0, 65535, PChar(aFileName+'F'));
  if FFileMapObj = 0 then
  begin
    if FMutex <> 0 then CloseHandle(FMutex);
    RaiseLastOSError;
  end;

  FIsNewFile := (GetLastError <> ERROR_ALREADY_EXISTS);

  FBaseAddress := MapViewOfFile(FFileMapObj, FILE_MAP_WRITE, 0 ,0, 0);
  if FBaseAddress = nil then
  begin
    if FMutex <> 0 then CloseHandle(FMutex);
    UnMapViewOfFile(FBaseAddress);
    RaiseLastOSError;
  end;

end;

destructor TFxMemoryFile.Destroy;
//==============================================================================
// 
//
// :  .                                                05.03.2011
// e-mail: albr@ukr.net; icq: 99258304; skype: albr78;
//==============================================================================
begin
  UnMapViewOfFile(FBaseAddress);
  CloseHandle(FFileMapObj);
  if FMutex <> 0 then CloseHandle(FMutex);

  inherited;
end;

procedure TFxMemoryFile.GetList(aStrings: TStrings);
//==============================================================================
//     . ( + .      .)
//
// :  .                                                05.03.2011
// e-mail: albr@ukr.net; icq: 99258304; skype: albr78;
//==============================================================================
begin
  if WaitForSingleObject(FMutex, INFINITE) = WAIT_OBJECT_0 then
  try
    aStrings.Text := PChar(FBaseAddress);
    StrPCopy(FBaseAddress, '');
  finally
    ReleaseMutex(FMutex);
  end else
    RaiseLastOsError();
end;

procedure TFxMemoryFile.LoadList(aStrings: TStrings);
//==============================================================================
//     . (.     .)
//
// :  .                                                05.03.2011
// e-mail: albr@ukr.net; icq: 99258304; skype: albr78;
//==============================================================================
begin
  if WaitForSingleObject(FMutex, INFINITE) = WAIT_OBJECT_0 then
  try
    aStrings.Text := PChar(FBaseAddress);
  finally
    ReleaseMutex(FMutex);
  end else
    RaiseLastOsError();
end;

procedure TFxMemoryFile.SaveList(aStrings: TStrings);
//==============================================================================
//     .
//
// :  .                                                05.03.2011
// e-mail: albr@ukr.net; icq: 99258304; skype: albr78;
//==============================================================================
begin
  if WaitForSingleObject(FMutex, INFINITE) = WAIT_OBJECT_0 then
  try
    StrPCopy(FBaseAddress, aStrings.text);
  finally
    ReleaseMutex(FMutex);
  end else
    RaiseLastOsError();
end;

end.
