unit uReader;
//==============================================================================
//      MT4 +    
//
// :  .                                                05.03.2011
// e-mail: albr@ukr.net; icq: 99258304; skype: albr78;
//==============================================================================

interface
uses
  Classes, SysUtils, uFxMemoryManager;

type
  TReader = class
  private
    FMemFile: TFxMemoryFile;
    FMemDLL: TFxMemoryFile;
    FMemDLL2: TFxMemoryFile;
    FList: TStringList;
    FOrderCount: integer;
    function GetValues(Name: string): string;
    function GetOrderProfit(OrderIndex: integer): double;
    function GetOrderTicket(OrderIndex: integer): integer;
    function GetOrderType(OrderIndex: integer): string;
    function GetOrderLots(OrderIndex: integer): double;
    function GetOrderSymbol(OrderIndex: integer): string;
    function GetOrderTotal(Currency: string): double;
    procedure SetValues(Name: string; const Value: string);
  public
    FList2: TStringList;
    constructor Create();
    destructor Destroy(); override;
    procedure Update;

    procedure ExecuteOrder(aOrder: string);
  public
    property Values[Name: string]:string read GetValues write SetValues;
    property OrderProfit[OrderIndex: integer]: double read GetOrderProfit;
    property OrderTicket[OrderIndex: integer]: integer read GetOrderTicket;
    property OrderType[OrderIndex: integer]: string read GetOrderType;
    property OrderLots[OrderIndex: integer]: double read GetOrderLots;
    property OrderSymbol[OrderIndex: integer]: string read GetOrderSymbol;
    property OrderCount: integer read FOrderCount;
    property OrderTotal[Currency: string]: double read GetOrderTotal;
  end;
implementation

{ TReader }
constructor TReader.Create;
//==============================================================================
// 
//
// :  .                                                05.03.2011
// e-mail: albr@ukr.net; icq: 99258304; skype: albr78;
//==============================================================================
begin
  FList := TStringList.Create;
  FList2 := TStringList.Create;
  FMemFile := TFxMemoryFile.Create('MAINFXMONITOR');
  FMemDLL := TFxMemoryFile.Create('DLLFXMONITOR');
  FMemDLL2 := TFxMemoryFile.Create('DLLFXMONITOR2');
end;

destructor TReader.Destroy;
//==============================================================================
// 
//
// :  .                                                05.03.2011
// e-mail: albr@ukr.net; icq: 99258304; skype: albr78;
//==============================================================================
begin
  FList.Free;
  FList2.Free;
  FMemFile.Free;
  FMemDLL.Free;
  FMemDLL2.Free;
  inherited;
end;

procedure TReader.ExecuteOrder(aOrder: string);
//==============================================================================
//     
//
// :  .                                                05.03.2011
// e-mail: albr@ukr.net; icq: 99258304; skype: albr78;
//==============================================================================
var
  fOrders: TStrings;
begin
  fOrders := TStringList.Create;
  try
    FMemFile.GetList(fOrders);
    fOrders.Add(aOrder);
    FMemFile.SaveList(fOrders);
  finally
    fOrders.Free;
  end;
end;

function TReader.GetOrderLots(OrderIndex: integer): double;
//==============================================================================
//    
//
// :  .                                                05.03.2011
// e-mail: albr@ukr.net; icq: 99258304; skype: albr78;
//==============================================================================
var
  Code: integer;
begin
  Val(Values[format('[ORDER_%d_LOTS]',[OrderIndex])], result, Code);

  if Code <> 0 then result := 0;
end;

function TReader.GetOrderProfit(OrderIndex: integer): double;
//==============================================================================
//     
//
// :  .                                                05.03.2011
// e-mail: albr@ukr.net; icq: 99258304; skype: albr78;
//==============================================================================
var
  Code: integer;
begin
  Val(Values[format('[ORDER_%d_PROFIT]',[OrderIndex])], result, Code);

  if Code <> 0 then
    result := 0;
end;

function TReader.GetOrderSymbol(OrderIndex: integer): string;
//==============================================================================
//  
//
// :  .                                                05.03.2011
// e-mail: albr@ukr.net; icq: 99258304; skype: albr78;
//==============================================================================
begin
  if OrderIndex = -1 then
    result := '' else
    result := Values[format('[ORDER_%d_SYMBOL]',[OrderIndex])];
end;

function TReader.GetOrderTicket(OrderIndex: integer): integer;
//==============================================================================
//  () 
//
// :  .                                                05.03.2011
// e-mail: albr@ukr.net; icq: 99258304; skype: albr78;
//==============================================================================
begin
  if not TryStrToInt(Values[format('[ORDER_%d_TICKET]',[OrderIndex])], result) then
    result := 0;
end;

function TReader.GetOrderTotal(Currency: string): double;
//==============================================================================
//  - 
//
// :  .                                                05.03.2011
// e-mail: albr@ukr.net; icq: 99258304; skype: albr78;
//==============================================================================
var
  i: integer;
  lsOrder: string;
begin
  Result := 0;

  if Currency <> '' then
    for i := 0 to FOrderCount - 1 do
    begin
      if OrderSymbol[i] = Currency then
      begin
        lsOrder := OrderType[i];
        if lsOrder = 'SELL' THEN
          Result := ROUND(Result*100 - OrderLots[i]*100)/100 else
        if lsOrder = 'BUY' THEN
          Result := ROUND(Result*100 + OrderLots[i]*100)/100;
      end;
    end;
end;

function TReader.GetOrderType(OrderIndex: integer): string;
//==============================================================================
//  
//
// :  .                                                05.03.2011
// e-mail: albr@ukr.net; icq: 99258304; skype: albr78;
//==============================================================================
begin
  result := Values[format('[ORDER_%d_TYPE]',[OrderIndex])];
end;

function TReader.GetValues(Name: string): string;
//==============================================================================
//    
//
// :  .                                                05.03.2011
// e-mail: albr@ukr.net; icq: 99258304; skype: albr78;
//==============================================================================
begin
  try
    Result := FList.Values[Name];
  except
    Result := '';
  end;
end;

procedure TReader.SetValues(Name: string; const Value: string);
//==============================================================================
//    (,    )
//
// :  .                                                05.03.2011
// e-mail: albr@ukr.net; icq: 99258304; skype: albr78;
//==============================================================================
begin
  try
    FList.Values[Name] := Value;
  except

  end;
end;

procedure TReader.Update;
//==============================================================================
//    
//
// :  .                                                05.03.2011
// e-mail: albr@ukr.net; icq: 99258304; skype: albr78;
//==============================================================================
begin
  FMemDLL.LoadList(FList);
  FMemDLL2.GetList(FList2);
  TryStrToInt(Values['[ORDERS]'], FOrderCount);
end;

end.
